
from log4py import Logger
import sys
import os
import time
import rpm
import packageSack
import packageObject
import repoMDObject
import mdUtils
import mdErrors


class UnresolvedDep:
    def __init__(self, pkg, reqstr):
        self.cat = Logger().get_instance(self)
        self.cat.debug("start")
        self.pkg = pkg
        self.reqstr = reqstr

class Dep:
    def __init__(self):
        self.cat = Logger().get_instance(self)
        self.cat.debug("start")
        self.pkgSack = packageSack.XMLPackageSack(packageObject.RpmXMLPackageObject)

    def addBaseRepo(self, location, process):
        self.cat.debug("location is " + location)
        repomdxmlfile = os.path.join(location, 'repodata/repomd.xml')
        try:
            repodata = repoMDObject.RepoMD(location, repomdxmlfile)
        except mdErrors.RepoMDError, e:
            print >> sys.stderr, e
            sys.exit(1)
    
        (pbase, phref) = repodata.primaryLocation()
        (fbase, fhref) = repodata.filelistsLocation()
        (obase, ohref) = repodata.otherLocation()
    
        # now add every primary and every filelist file
        processlist = [phref, fhref]
        for file in processlist:
            self.cat.debug('importing ' + str(file) + ' from ' + str(location))
            complete = location + '/' + file
            try:
                self.pkgSack.addFile(location, complete, process)
            except mdErrors.PackageSackError, e:
                print >> sys.stderr, e
                sys.exit(1)

    def checkPackageByName(self, pkgName):
        self.cat.debug('start, pkgName: ' + pkgName)
        retval = []
        for pkg in self.pkgSack.searchNevra(pkgName):
            retval = retval + self.checkPackage(pkg)
        return retval
            
            
    def checkPackage(self, pkg):
        self.cat.debug('start, pkg: ' + str(pkg))
        retval = []
        for reqtup in pkg.returnPrco('requires'):
            (reqn, reqf, (reqe,reqv,reqr)) = reqtup
            reqstr = 'reqn: ' + str(reqn) + ', reqf: ' + str(reqf) + ', reqe: ' + str(reqe) + ', reqv: ' + str(reqv) + ', reqr: ' + str(reqr)
            self.cat.debug('reqstr: ' + reqstr)
            # rpmlib deps should be handled on their own
            if reqn[:6] == 'rpmlib':
                continue
            # kill self providers, too
            if pkg.checkPrco('provides', reqtup):
                continue
          
            # get a list of all pkgs that match the reqn
            providers = self.pkgSack.searchProvides(reqn)
            if len(providers) == 0:
                self.cat.debug('unresolved dependency: ' + reqstr)
                retval.append(UnresolvedDep(pkg,reqstr))
        return retval
                    
    def checkAllPackages(self, repoid=None):
        self.cat.debug('start, repoid=' + repoid)
        # some checks
        #for temprepoid in self.getRepoids():
        #    self.cat.debug('check, temprepoid="' + temprepoid + '" len=' + str(len(self.pkgSack.simplePkgList(temprepoid))))
        retval = []
        if repoid == None:
            for pkg in self.pkgSack.returnPackages():
                retval = retval + self.checkPackage(pkg)
        else:
            for pkg in self.pkgSack.returnPackages(repoid):
                retval = retval + self.checkPackage(pkg)
        return retval

    def getRepoids(self):
        return self.pkgSack.pkgsByRepo.keys()
            

# because this class will be used within a larger program 
# without console output: no fancy progress bar
def process(current, total):
    return

#mydep = Dep()
# fc2os, fc2dag and fc2dries are directories which contain a repodata
# directory with all the xml files
#mydep.addBaseRepo('fc2os', process)
#mydep.addBaseRepo('fc2dag', process)
#mydep.addBaseRepo('fc2dries', process)
# print mydep.getRepoids()
# mydep.checkPackage('gambas')
# liquidwar needs allegro which is in newrpms
# mydep.checkPackage('liquidwar')
# mydep.checkAllPackages()


if len(sys.argv) < 2:
    print 'rpmdep /path/to/repo /path/to/repo ..'
    print 'Each dir should contain a repodata dir with a repomd.xml file'
    print 'The dependencies of the last repo are checked.'
    sys.exit(1)


mydep = Dep();
repodirs = sys.argv[1:]
lastrepo = ""
for repo in repodirs:
    mydep.addBaseRepo(repo,process)
    lastrepo = repo
unrdeplist = mydep.checkAllPackages(lastrepo)
for unrdep in unrdeplist:
    print "package " + str(unrdep.pkg) + " needs " + str(unrdep.reqstr)
